#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "fitsdebug.h"
#include "utilities.h"



int fd_set_fptr(fd_file *file, fitsfile *fptr) {

  file->fptr = fptr;

  return 1;

}

fitsfile *fd_get_fptr(fd_file *file) {

  return file->fptr;

}

/* --------------------------------- */

int fd_set_filename(fd_file *file, char filename[MAXSTRLEN]) {

  strcpy(file->filename,filename);

  return 1;

}

char *fd_get_filename(fd_file *file) {

  return file->filename;

}

/* --------------------------------- */

int fd_set_tablename(fd_file *file, char *tablename) {

  if (tablename != NULL) {
    strcpy(file->tablename,tablename);
    return 1;
  } else {
    return 0;
  }

}

char *fd_get_tablename(fd_file *file) {

  return file->tablename;

}

/* --------------------------------- */

int fd_set_hdu(fd_file *file, int hdu) {

  file->hdu = hdu;

  return 1;

}

int fd_get_hdu(fd_file *file) {

  return file->hdu;

}

/* --------------------------------- */

int fd_set_tabletype(fd_file *file, int tabletype) {

  if ((tabletype == ASCII_TBL) || 
      (tabletype == BINARY_TBL)) {
    file->tabletype = tabletype;
    return 1;
  } else {
    return 0;
  }

}

int fd_get_tabletype(fd_file *file) {

  int ttype;

  ttype = file->tabletype;

  if ((ttype == ASCII_TBL) || 
      (ttype == BINARY_TBL)) {
    return ttype;
  } else {
    return -1;
  }

}

/* --------------------------------- */

int fd_set_ncol(fd_file *file, int ncol) {

  file->ncol = ncol;

  return 1;

}

int fd_set_ncol_ap(fd_file *file, va_list ap) {

  int ncol = 0;

  while (va_arg(ap, char *)) {   /* null pointer ends list */
    ncol++;
  }

  if (ncol%3 != 0) {
    if (FD_VERBOSE >= 1) printf("Error (fd,fd_set_ncol_ap) : ncol is nor a multiple of 3\n");
    return 0;
  } else {
    file->ncol = ncol/3;
    return 1;
  }

}

int fd_get_ncol(fd_file *file) {

  return file->ncol;

}

/* --------------------------------- */

int fd_set_nrow(fd_file *file, long nrow) {

  file->nrow = nrow;

  return 1;

}

long fd_get_nrow(fd_file *file) {

  return file->nrow;

}
/* --------------------------------- */

int fd_set_tformi(fd_file *file, int i, char *tform) {
  
  int status = 0;


  /* Transform the raw format into the fits accepted format  */
  /* allocate memory and store the value                     */
    
  if (file->tabletype == BINARY_TBL) 
    fd_get_typecode_binary(&file->typecode[i], tform, &status);
  else
    fd_get_typecode_ascii(&file->typecode[i], tform, &status);

  if (status != FD_STAT_ALLOK) {
    return 0;
  }
    
  file->tform[i] = mem_malloc_safe((strlen(file->typecode[i].type_code)+1)*
			       sizeof(char));
  strcpy(file->tform[i],file->typecode[i].type_code);

  return 1;

}

char *fd_get_tformi(fd_file *file, int i) {
  
  char *tmp;

  tmp = mem_malloc_safe((strlen(file->tform[i])+1)*sizeof(char));

  strcpy(tmp,file->tform[i]);

  return tmp;

}

/* --------------------------------- */

int fd_set_ttypei(fd_file *file, int i, char *ttype) {
  
  file->ttype[i] = mem_malloc_safe((strlen(ttype)+1)*sizeof(char));

  strcpy(file->ttype[i],ttype);

  return 1;

}

char *fd_get_ttypei(fd_file *file, int i) {
  
  char *tmp;

  tmp = mem_malloc_safe((strlen(file->ttype[i])+1)*sizeof(char));

  strcpy(tmp,file->ttype[i]);

  return tmp;

}

/* --------------------------------- */

int fd_set_tuniti(fd_file *file, int i, char *tunit) {
  
  file->tunit[i] = mem_malloc_safe((strlen(tunit)+1)*sizeof(char));

  strcpy(file->tunit[i],tunit);

  return 1;

}

char *fd_get_tuniti(fd_file *file, int i) {
  
  char *tmp;

  tmp = mem_malloc_safe((strlen(file->tunit[i])+1)*sizeof(char));

  strcpy(tmp,file->tunit[i]);

  return tmp;

}

/* --------------------------------- */

int fd_set_flush(fd_file *file, int flush) {

  if ((flush == FD_FLUSH_ON) ||
      (flush == FD_FLUSH_OFF)) {
    file->flush = flush;
    return 1;
  } else {
    return 0;
  }

}

int fd_get_flush(fd_file *file) {

  int flush;

  flush = file-> flush;

  if ((flush == FD_FLUSH_ON) ||
      (flush == FD_FLUSH_OFF)) {
    return flush;
  } else {
    return -1;
  }

}

/* --------------------------------- */

int fd_set_supersede(fd_file *file, int supersede) {

  if ((supersede == FD_SUPERSEDE_ON) ||
      (supersede == FD_SUPERSEDE_OFF)) {
    file->supersede = supersede;
    return 1;
  } else {
    return 0;
  }

}

int fd_get_supersede(fd_file *file) {

  int supersede;

  supersede = file->supersede;

  if ((supersede == FD_SUPERSEDE_ON) ||
      (supersede == FD_SUPERSEDE_OFF)) {
    return supersede;
  } else {
    return -1;
  }

}

/* --------------------------------- */

int fd_set_file_status(fd_file *file, int file_status) {

  if ((file_status == FD_FILE_OPEN) ||
      (file_status == FD_FILE_CLOSED) ||
      (file_status == FD_FILE_UNKNOWN)) {
    file->file_status = file_status;
    return 1;
  } else {
    return 0;
  }
    
}

int fd_get_file_status(fd_file *file) {

  return file->file_status;

}

/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */
/* --------------------------------- */


