#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* Rewritten from Numerical recipes.  Free version must be obtained */
/* before release! */
 
void prep_spline(double x[], double y[], int n, double y2[]) {

  int i,k;
  double p, qn, sig, un, *u;
  
  u = (double *) malloc(n*sizeof(double));

  y2[0]=0.0;
  u[0]=0.0;

  for (i=1;i<n-1;i++) {
    sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
    p=sig*y2[i-1]+2.0;
    y2[i]=(sig-1.0)/p;
    u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
  }

  qn=0.0;
  un=0.0;
  
  y2[n-1]=(un-qn*u[n-2])/(qn*y2[n-2]+1.0);

  for (k=n-2;k>0;k--)  y2[k]=y2[k]*y2[k+1]+u[k];

  free(u);
  return;
}

double calc_spline(double xa[], double ya[], double y2a[], int n, double x) {

  int klo,khi,k;
  double h,b,a;
  double result;

  klo=0;
  khi=n-1;

  while (khi-klo > 1) {
    k=(khi+klo) >> 1;
    if (xa[k] > x) khi=k;
    else klo=k;
  }
  h=xa[khi]-xa[klo];
  if (h == 0.0) {
    fprintf(stderr,"calc_spline: Bad x array input\n");
    abort();
  }

  a=(xa[khi]-x)/h;
  b=(x-xa[klo])/h;
  result=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;

  return result;
}
