#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "apec.h"
#include "messages.h"

void fits_printerror(char *process, int status);
char *roman_element(int Z, int J);

void append_chksum(struct CHKSUM *top, fitsfile *fptr) {

  int chdu, status=0;
  char comment[80];
  struct CHKSUM *ptr;
  char *typename[6] = {"ionization/recombination","energy levels",
		       "emission lines","electron collisions",
		       "proton collisions","dielectronic satellite lines"};

  ptr = top;

  /* Store our current position */
  fits_get_hdu_num(fptr, &chdu);

  /* Go to the initial, primary array */
  if (fits_movabs_hdu(fptr, 1, NULL, &status)) {
    fits_printerror("moving to PRIMARY hdu.",status);
  }
  
  /* Start storing information */
  while (ptr != NULL) {
    sprintf(comment, "Checksum for %s:%s",roman_element(ptr->Z,ptr->rmJ-1), 
	    typename[ptr->type-1]);
    if (fits_write_key(fptr, TSTRING, ptr->keyname, ptr->keyval,
		       comment, &status)) {
      fits_printerror("writing checksum keyword", status);
    }
    ptr = ptr->next;
  }

  /* And now jump back to the most recent HDU */
  if (fits_movabs_hdu(fptr, chdu, NULL, &status)) {
    fits_printerror("returning to current HDU.",status);
  }
}
