#include <stdlib.h>
#include <stdio.h>

int create_parameter_file(char *filename) {

  FILE *fp;

  fp = fopen(filename, "w");

  if (fp == NULL) {
    printf("apec: Error, can't write parameter file %s", filename);
    return(-1);
  }
  
  fprintf(fp,"FileMap,s,a,\"$ATOMDB/filemap\",,,\"Parameter file listing input atomic data\"\n");
  fprintf(fp,"RequireChecksum,b,h,yes,,,\"Checksums required in atomic data files?\"\n");
  fprintf(fp,"Ionization,s,h,\"CIE\",\"CIE|NEI\",,\"Collisional Equilibrium or Non-equilibrium\"\n");
  fprintf(fp,"UseIonBalanceTable,b,h,yes,,,\"Use pre-calculated ionization balance table\"\n");
  fprintf(fp,"IonBalanceTable,s,a,\"$ATOMDB/APED/ionbal/MM98_ionbal.fits\",,,\"Ionization balance table\"\n");
  fprintf(fp,"Abundances,s,a,\"AG89\",\"Allen|AG89|GA88|Feldman\",,\"Abundances\"\n");
  fprintf(fp,"MinEpsilon,r,h,1.e-20,,,\"Minimum emissivity for detailed line output, in ph cm^3/s\"\n");
  fprintf(fp,"TempUnits,s,a,\"kelvin\",\"kelvin|keV\",,\"Units for temperature values\"\n");
  fprintf(fp,"LinearTemp,b,a,no,,,\"Use linear spacing for temperatures (if no, use logarithmic)\"\n");
  fprintf(fp,"TempStart,r,a,4,,,\"Minimum temperature to calculate (in dex if LinearTemp=no)\"\n");
  fprintf(fp,"TempStep,r,a,0.1,,,\"Temperature step size (in dex if LinearTemp=no)\"\n");
  fprintf(fp,"NumTemp,i,a,50,1,,\"Number of temperatures to calculate\"\n");
  fprintf(fp,"LinearDens,b,a,no,,,\"Use linear spacing for densities (if no use logarithmic)\"\n");
  fprintf(fp,"DensStart,r,a,0,,,\"Minimum density to calculate (in dex if LinearDens=no)\"\n");
  fprintf(fp,"DensStep,r,a,0.1,,,\"Density step size (in dex if LinearDens=no)\"\n");
  fprintf(fp,"NumDens,i,a,1,1,,\"Number of densities to calculate\"\n");
  fprintf(fp,"GridUnits,s,a,\"keV\",\"keV|angstrom\",,\"Continuum grid units\"\n");
  fprintf(fp,"LinearGrid,b,a,yes,,,\"Use linear spacing for grid (if no, use logarithmic)\"\n");
  fprintf(fp,"GridMinimum,r,a,0.01,1.e-10,,\"Minimum value for continuum grid\"\n");
  fprintf(fp,"GridMaximum,r,a,10.0,0,,\"Maximum value for continuum grid\"\n");
  fprintf(fp,"NumGrid,i,a,10000,100,100000,\"Number of bins in continuum grid\"\n");
  fprintf(fp,"IncAtoms,s,a,\"H,He,C,N,O,Ne,Mg,Al,Si,S,Ar,Ca,Fe,Ni\",,,\"Include all ions of listed atoms\"\n");
  fprintf(fp,"IncIons,s,a,\"\",,,\"Include listed ions only (ie, S XI,O VII,Fe XVII)\"\n");
  fprintf(fp,"RRC,b,h,yes,,,\"Calculate radiative recombination continua?\"\n");
  fprintf(fp,"DRSatellite,b,h,yes,,,\"Calculate dielectronic recombination satellites?\"\n");
  fprintf(fp,"EmissionLines,b,h,yes,,,\"Calculate emission lines?\"\n");
  fprintf(fp,"TwoPhoton,b,h,yes,,,\"Calculate Two-Photon Emission?\"\n");
  fprintf(fp,"Bremsstrahlung,b,h,yes,,,\"Calculate bremsstrahlung?\"\n");
  fprintf(fp,"BremsType,s,h,\"Kellogg\",\"Hummer|Kellogg|Relativisitic|None\",,\"Type of Bremsstrahlung calcuation\"\n");
  fprintf(fp,"OutputFileStem,s,a,\"apec_plasma\",,,\"Filename stem for all outputs\"\n");
  fprintf(fp,"OutputType,s,a,\"small\",\"all|full|small\",,\"Files to be created\"\n");
  fprintf(fp,"OutputPerIon,b,h,no,,,\"Output lines without ionization balance included\"\n");
  fprintf(fp,"DebugFormat,s,h,\"fits\",\"fits|ascii|both\",,\"Output debugging data as fits or ascii files\"\n");
  fprintf(fp,"DebugFitsFormat,s,h,\"binary\",\"binary|ascii\",,\"Output FITS file table type\"\n");
  fprintf(fp,"DebugEmission,b,h,no,,,\"Debugging emission line data\"\n");
  fprintf(fp,"DebugIonBalance,b,h,no,,,\"Debugging emission line data\"\n");
  fprintf(fp,"DebugTable,b,h,no,,,\"Debugging table data\"\n");
  fprintf(fp,"DebugIonization,b,h,no,,,\"Debugging ionization data\"\n");
  fprintf(fp,"DebugRecombination,b,h,no,,,\"Debugging recombination data\"\n");
  fprintf(fp,"DebugTransition,b,h,no,,,\"Debugging transition data\"\n");
  fprintf(fp,"DebugRates,b,h,no,,,\"Debugging rate data\"\n");
  fprintf(fp,"DebugSatellite,b,h,no,,,\"Debugging satellite line data\"\n");
  fprintf(fp,"DebugTwophoton,b,h,no,,,\"Debugging two-photon emission data\"\n");
  fprintf(fp,"DebugLevelPopulation,b,h,no,,,\"Debugging level population data\"\n");
  fprintf(fp,"DebugCompression,b,h,no,,,\"Debugging compression data\"\n");
  fprintf(fp,"mode,s,hl,\"hl\",\"ql|hl|q|h\",,\"mode for parameter file\"\n");

  if (fclose(fp)) { 
    printf("apec: Error, can't write parameter file %s", filename);
    return(-1);
  }
  return 0;
}
