#include <stdlib.h>
#include <stdio.h>
#include <fitsio.h>

#include "apec.h"

void fits_write_input(fitsfile *fptr, struct PARAMETERS *params);
void append_chksum(struct CHKSUM *chksum_top, fitsfile *fptr);
void create_chksum_arrays(int N_Elements, int *list_Z, 
			  ElementP *Atomic_data, struct CHKSUM **ptr);
void fits_spec_init(char *filehead, int filetype, int Nrows, fitsfile **fptr);
void init_fits_spectral_table(struct PARAMETERS *params, 
			      struct BIN_SPEC *spectrum);

void fits_open(struct PARAMETERS *params, ElementP *Atomic_data, 
	       struct BIN_SPEC *spectrum) {

  struct CHKSUM *chksum_top=NULL;
  int Nrows=0;
  Nrows = params->Ndens*params->Ntemp;

  /* Make checksum arrays */
  create_chksum_arrays(params->N_Elements, params->list_Z, 
		       Atomic_data, &chksum_top);

  /* Open up all fits files, as needed */

  if (params->do_line) {
    fits_spec_init(params->fn_linespec, 0, Nrows, &(params->fptr_line));
    append_chksum(chksum_top, params->fptr_line);
    fits_write_input(params->fptr_line, params); 
  } else {
    params->fptr_line = NULL;
  }
  
  if (params->do_bincont) {
    fits_spec_init(params->fn_bincontspec, 1, Nrows, &(params->fptr_bincont));
    append_chksum(chksum_top, params->fptr_bincont);
    fits_write_input(params->fptr_bincont, params); 
  } else {
    params->fptr_bincont = NULL;
  }

  if (params->do_coco) {
    fits_spec_init(params->fn_cocospec, 2, Nrows, &(params->fptr_coco));
    append_chksum(chksum_top, params->fptr_coco);
    fits_write_input(params->fptr_coco, params); 
  } else {
    params->fptr_coco = NULL;
  }

  if (params->do_comp) {
    fits_spec_init(params->fn_compspec, 3, Nrows, &(params->fptr_comp));
    append_chksum(chksum_top, params->fptr_comp);
    fits_write_input(params->fptr_comp, params); 
  } else {
    params->fptr_comp = NULL;
  }
  
  if (params->do_xspectable) {
    init_fits_spectral_table(params, spectrum);
    fits_write_input(params->fptr_xspectable, params); 
  } else {
    params->fptr_xspectable = NULL;
  }

}
