#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <fitsio.h>

#include "apec.h"
#include "messages.h"

#define ARNAUD 1.e14
#define OGIP_TABLE_MAGIC 6
#define TABLE_LIN_INTERP 0
#define TABLE_LOG_INTERP 1
#ifndef  max
#define max(a, b)       ((a) < (b) ? (b) : (a))
#endif  /* max */

#define MAXPARM 2
extern void wftbmd(char *outfil, char *infile, char *modlnm, char *modunt,
	    int nintpm, int naddpm, int qrdshf, int qaddtv, char *addnam,
	    float *addivl, char *intnam[], float *intivl, int *intntb,
	    long *intmth, int maxtab, float *inttab, int nenerg,
	    float *energy, fitsfile **fptr, int *ierr, int *status,
	    char **contxt);

void init_fits_spectral_table(struct PARAMETERS *params, 
			      struct BIN_SPEC *spectrum) {
  
  int i;
  int nintpm, naddpm, maxtab, nenerg, ierr=0, status=0;
  int *intntb;
  long *intmth;
  float *addivl, *inttab, *intivl, *energy;
  char *outfil, *infile, *modlnm, *modunt, *addnam, *intnam[MAXPARM];
  char *contxt;
  int qaddtv, qrdshf;
  
  if (params->fn_xspectable == NULL) {
    outfil = "APEC_Model.fits";
  } else {
    outfil = params->fn_xspectable;
  }
  infile = "DUMMY";
  if (params->modelname == NULL) {
    modlnm = "APEC Table Model";
  } else {
    modlnm = params->modelname;
  }
  modunt = "photons/cm^2/s";
  if (params->Ndens>1) {
    nintpm = 2;
  } else {
    nintpm = 1;
  }
  naddpm = 0;
  qrdshf = 0;
  qaddtv = 1;
  addnam = NULL;
  addivl = NULL;
  
  intivl = (float *) malloc(OGIP_TABLE_MAGIC*nintpm*sizeof(float));
  intntb = (int *) malloc(OGIP_TABLE_MAGIC*nintpm*sizeof(int));
  intmth = (long *) malloc(nintpm*sizeof(long));
  intnam[0] = (char *) malloc(MAXSTRLEN*sizeof(char));
  sprintf(intnam[0],"kT(keV)");
  intivl[0] = (float) (params->temperature)[params->Ntemp/2]*KBOLTZ;
  if (params->Ntemp > 1) {
    intivl[1] = (float) ((params->temperature)[1] -
			 (params->temperature)[0])*KBOLTZ;
  } else {
    intivl[1] = 0.0;
  }
  intivl[2] = (float) ((params->temperature)[0])*KBOLTZ;
  intivl[3] = (float) ((params->temperature)[0])*KBOLTZ;
  intivl[4] = (float)((params->temperature)[params->Ntemp-1])*KBOLTZ;
  intivl[5] = (float)((params->temperature)[params->Ntemp-1])*KBOLTZ;
  intmth[0] = TABLE_LOG_INTERP;
  intntb[0] = params->Ntemp;
  maxtab = max(params->Ntemp,params->Ndens);
  inttab = (float *) malloc(nintpm*maxtab*sizeof(float));
  for (i=0;i<params->Ntemp;i++) {
    inttab[maxtab*0 + i] = ((params->temperature)[i])*KBOLTZ;
  }

  if (params->Ndens>1) {
    intnam[1] = (char *) malloc(MAXSTRLEN*sizeof(char));
    sprintf(intnam[1],"Density");
    intivl[6] = (float) (params->density)[params->Ndens/2];
    intivl[7] = (float) (params->density)[1] - (params->density)[0];
    intivl[8] = (float) (params->density)[0];
    intivl[9] = (float) (params->density)[0];
    intivl[10]= (float) (params->density)[params->Ndens-1];
    intivl[11]= (float) (params->density)[params->Ndens-1];
    intmth[1] = TABLE_LOG_INTERP;
    intntb[1] = params->Ndens;  
    for (i=0;i<params->Ndens;i++) {
      inttab[maxtab*1 + i] = (params->density)[i];
    }
  }

  nenerg = spectrum->Nbin+1;
  energy = (float *) malloc(nenerg*sizeof(float));
  for (i=0;i<nenerg;i++) energy[i] = spectrum->bin_edge[i];
  
  /* And now, write out the header. */

  wftbmd(outfil, infile, modlnm, modunt, nintpm, naddpm, qrdshf, qaddtv,
	 addnam, addivl, intnam, intivl, intntb, intmth, maxtab, inttab,
	 nenerg, energy, &(params->fptr_xspectable), &ierr, &status, &contxt);
  
  if ((status != 0)||(ierr != 0)) {
    message("fits_spectral_table","Error writing header of FITS table file.");
    message("fits_spectral_table","Status is %d",status);
    message("fits_spectral_table","Error code is %d",ierr);
    errmess("fits_spectral_table","Context is %s",contxt);
  }

  free(energy);
  free(inttab);
  free(intivl);
  free(intntb);
  free(intmth);
  free(intnam[0]);
  if (params->Ndens>1) free(intnam[1]);
}
			      
void write_fits_spectral_table(struct PARAMETERS *params, 
			       fitsfile *fptr_spectral, int iTemp, int iDens, 
			       struct BIN_SPEC *spectrum){

  int status=0;
  long firstrow, firstelem, nelements;
  float *data, paramdata[2];
  float Ebin;
  int i, iZ, Z;

  firstrow = params->Ntemp*iDens + iTemp + 1;
  firstelem = 1;

  /* message("fits_spectral_table","Ntemp = %d, iTemp = %d, iDens = %d",
	  params->Ntemp, iTemp, iDens); */

  /* Parameters: Temperature & Density (1st column) */
  paramdata[0] = (float) ((params->temperature)[iTemp])*KBOLTZ;
  nelements = 1;

  if (params->Ndens>1) {
    paramdata[1] = (float) (params->density)[iDens];
    nelements = 2;
  }
  
  if (fits_write_col(fptr_spectral, TFLOAT, 1, firstrow, firstelem, nelements,
		 paramdata, &status)) {
    message("fits_spectral_table","Error writing parameter data, row %d",
	    firstrow);
  }

  /* Spectrum (2rd column) */
  nelements = spectrum->Nbin;
  data = (float *) malloc(nelements*sizeof(float));
  for (i=0;i<nelements;i++) {
    Ebin = ( ( (spectrum->bin_edge)[i] + (spectrum->bin_edge)[i+1] ) / 2.0);
    Ebin *= ERG_KEV;
    for (iZ=0;iZ<params->N_Elements;iZ++) {
      Z = params->list_Z[iZ];
      if (iZ==0) {
	data[i] = (float) ARNAUD*(((spectrum->bin_emission[Z])[i]));
      } else {
	data[i] += (float) ARNAUD*(((spectrum->bin_emission[Z])[i]));
      }
    }
    /* message("fits_spectral_table","kT = %f, Brems[%d] at E = %7.2e : %7.2e",
       paramdata[0],i,Ebin/ERG_KEV,data[i]); */
  }

  if (fits_write_col(fptr_spectral, TFLOAT, 2, firstrow, firstelem,
		     nelements, data, &status)) {
    message("fits_spectral_table","Error writing spectral data column, row %d",
	    firstrow);
  }
  
  free(data);
}

void close_fits_spectral_table(struct PARAMETERS *params) {

  int status=0;

  if (fits_close_file(params->fptr_xspectable, &status) ) {
    message("fits_spectral_table","Cannot close fits table model file.");
  }
}
