%
% General purpose SLang routines
%
%
%
%

uses("SL-utilities");
use_namespace("Utils");



%-----------------------------------------------

define array_from_args() {

   if (_NARGS == 0) {
      error("Utils->array_from_args() called without arguments");
   }

   variable test = ();
   variable array = @Array_Type(typeof(test),_NARGS);
   variable i;
   array[_NARGS-1] = test;

   for (i = _NARGS-2; i >= 0; i--) array[i] = ();

   return array;

}

define sleep (sec) {

   % Sleep for "sec" seconds
   %

   % start timing
   tic();

   while(sec < toc()) {
      continue;
   }

}


define print_typeof(obj) {
   
   %
   % Returns a string representation of the type of "obj"
   %

   switch(typeof(obj))
   { case Char_Type     : return "signed character"; }
   { case UChar_Type    : return "unsigned character"; }
   { case Short_Type    : return "short integer"; }
   { case UShort_Type   : return "unsigned short integer"; }
   { case Integer_Type  : return "integer"; }
   { case UInteger_Type : return "unsigned integer"; }
   { case Long_Type     : return "long integer"; }
   { case ULong_Type    : return "unsigned long integer"; }
   { case Float_Type    : return "single precision real"; }
   { case Double_Type   : return "double precision real"; }
   { case Complex_Type  : return "complex"; }
   { case String_Type   : return "string"; }
   { case BString_Type  : return "binary structure"; }
   { case Struct_Type   : return "structure"; }
   { case Ref_Type      : return "reference"; }
   { case Null_Type     : return "NULL"; }
   { case Array_Type    : return "array"; }
   { case DataType_Type : return "data type"; }

}

define print__typeof(obj) {
   
   %
   % Returns a string representation of the type of "obj"
   %

   switch(_typeof(obj))
   { case Char_Type     : return "signed character"; }
   { case UChar_Type    : return "unsigned character"; }
   { case Short_Type    : return "short integer"; }
   { case UShort_Type   : return "unsigned short integer"; }
   { case Integer_Type  : return "integer"; }
   { case UInteger_Type : return "unsigned integer"; }
   { case Long_Type     : return "long integer"; }
   { case ULong_Type    : return "unsigned long integer"; }
   { case Float_Type    : return "single precision real"; }
   { case Double_Type   : return "double precision real"; }
   { case Complex_Type  : return "complex"; }
   { case String_Type   : return "string"; }
   { case BString_Type  : return "binary structure"; }
   { case Struct_Type   : return "structure"; }
   { case Ref_Type      : return "reference"; }
   { case Null_Type     : return "NULL"; }
   { case Array_Type    : return "array"; }
   { case DataType_Type : return "data type"; }

}

define _print_typeof(obj) {
   
   %
   % Returns a string representation of the type of "obj"
   %

   switch(typeof(obj))
   { case Char_Type     : return "Char_Type"; }
   { case UChar_Type    : return "UChar_Type"; }
   { case Short_Type    : return "Short_Type"; }
   { case UShort_Type   : return "UShort_Type"; }
   { case Integer_Type  : return "Integer_Type"; }
   { case UInteger_Type : return "UInteger_Type"; }
   { case Long_Type     : return "Long_Type"; }
   { case ULong_Type    : return "ULong_Type"; }
   { case Float_Type    : return "Float_Type"; }
   { case Double_Type   : return "Double_Type"; }
   { case Complex_Type  : return "Complex_Type"; }
   { case String_Type   : return "String_Type"; }
   { case BString_Type  : return "BString_Type"; }
   { case Struct_Type   : return "Struct_Type"; }
   { case Ref_Type      : return "Ref_Type"; }
   { case Null_Type     : return "Null_Type"; }
   { case Array_Type    : return "Array_Type"; }
   { case DataType_Type : return "DataType_Type"; }

}

define _print__typeof(obj) {
   
   %
   % Returns a string representation of the type of "obj"
   %

   switch(_typeof(obj))
   { case Char_Type     : return "Char_Type"; }
   { case UChar_Type    : return "UChar_Type"; }
   { case Short_Type    : return "Short_Type"; }
   { case UShort_Type   : return "UShort_Type"; }
   { case Integer_Type  : return "Integer_Type"; }
   { case UInteger_Type : return "UInteger_Type"; }
   { case Long_Type     : return "Long_Type"; }
   { case ULong_Type    : return "ULong_Type"; }
   { case Float_Type    : return "Float_Type"; }
   { case Double_Type   : return "Double_Type"; }
   { case Complex_Type  : return "Complex_Type"; }
   { case String_Type   : return "String_Type"; }
   { case BString_Type  : return "BString_Type"; }
   { case Struct_Type   : return "Struct_Type"; }
   { case Ref_Type      : return "Ref_Type"; }
   { case Null_Type     : return "Null_Type"; }
   { case Array_Type    : return "Array_Type"; }
   { case DataType_Type : return "DataType_Type"; }

}


define member (obj, array) {

   if (_typeof(obj) != _typeof(array)) {
      message("In routine \"Utils->member(obj,array)\":"); 
      vmessage("  Data types of object (%s) and array (%s) do not match ", print__typeof(obj), print__typeof(array));
      return NULL;
   }


   variable x = where(array == obj);

   if (length(x) == 0) {
      return NULL;
   } else {
      return x[0];
   }

}


define check_nargs() {

   %
   % checks if _NARGS has the correct value
   %

   variable usage = "Usage: Utils->check_nargs(_NARGS,num1,num2,...)";


   if (_NARGS < 2) {
      message(usage);
      _pop_n(_NARGS);
      return NULL;
   }

   variable valid_nargs = Integer_Type[_NARGS-1];
   variable i, nargs;

   for (i = 0; i < _NARGS-1; i++) {
      valid_nargs[i] = ();
   }

   nargs = ();
   
   if (member(nargs,valid_nargs) == NULL) {
      return 0;
   } else { 
      return 1;
   }

}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

define assoc_length (assoc) {

   return length (assoc_get_keys (assoc));

}


define get_format (obj) {

   switch (typeof(obj)) 
   { case Char_Type     : return "%c"; }
   { case UChar_Type    : return "%c"; }
   { case Short_Type    : return "%d"; }
   { case UShort_Type   : return "%d"; }
   { case Integer_Type  : return "%d"; }
   { case UInteger_Type : return "%d"; }
   { case Long_Type     : return "%d"; }
   { case ULong_Type    : return "%d"; }
   { case Float_Type    : return "%e"; }
   { case Double_Type   : return "%e"; }
   { case Complex_Type  : error("No format available for Complex_Type"); }
   { case String_Type   : return "%s"; }
   { case BString_Type  : error("No format available for BString_Type"); }
   { case Struct_Type   : error("No format available for Struct_Type"); }
   { case Ref_Type      : error("No format available for Ref_Type"); }
   { case Null_Type     : error("No format available for Null_Type"); }
   { case Array_Type    : error("No format available for Array_Type"); }
   { case DataType_Type : error("No format available for DataType_Type"); }

}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

define equal (obj1, obj2) {

   variable type1 = typeof(obj1), type2 = typeof(obj2);

   if (type1 != type2) return 0;

   switch (type1) 
   { case Array_Type : return array_equal(obj1,obj2); }
   { case Struct_Type : return struct_equal(obj1,obj2); }
   { case Null_Type : return 1; }
   { return (obj1 == obj2); }

}

define array_equal (arr1, arr2) {

   variable usage = "Utils->array_equal(Array_Type array1, Array_Type array2)";

   if (orelse {Array_Type != typeof(arr1)} {Array_Type != typeof(arr2)}) {
      message(usage);
      return 0;
   }

   % Array contents have different types
   if (_typeof(arr1) != _typeof(arr2)) return 0;

   if (length(arr1) == length(arr2)) {
      % The array length is the same, now check the content.
      variable i;
      for (i = 0; i < length(arr1); i++) {
         if (arr1[i] != arr2[i]) return 0;
      }

      return 1;
   } else {
      % These are not the same arrays.
      return 0;
   }

}

define struct_equal (struct1, struct2) {

   % Compare two structures, return 1 if their contents are the same
   % return 0 otherwise.

   % test types of input
   %
   if (is_struct_type(struct1) == 0 or 
       is_struct_type(struct2) == 0) {
      variable type1 = _print_typeof(struct1), type2 = _print_typeof(struct2);
      verror("\nError: Utils->struct_equal(struct1,struct2): \n  Expect Struct_Type for struct1 and struct2\n  Received %s for struct1 and %s for struct2.",type1,type2);
   }

   variable fields1, fields2, i, n1, n2;

   %
   % Get structure field names.
   %
   fields1 = get_struct_field_names(struct1);
   fields2 = get_struct_field_names(struct2);
   
   % Compare number of structure fields
   %
   if (length(fields1) != length(fields2)) return 0;
   
   % Check if structure fields names are the same
   %
   i = 0;
   loop (length(fields1)) {
      if (fields1[i] != fields2[i]) return 0;
      i++;
   }

   % Check if structure field content is the same
   %
   i = 0;
   loop (length(fields1)) {
      !if (equal(get_struct_field(struct1,fields1[i]),
		 get_struct_field(struct2,fields2[i]))) return 0;
      i++;
   }

   return 1;

}

