#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>

#include "apec.h"
#include "_apec_slang.h"

/************************************************************************


**************************************************************************/



void free_emissivity_curve(Emissivity_Curve_Type *emiss_curve) {

  long i;

  if (NULL != emiss_curve->kT) free(emiss_curve->kT);
  if (NULL != emiss_curve->density) free(emiss_curve->density);
  for (i=0;i<emiss_curve->nT;i++) 
    if (NULL != emiss_curve->emissivity[i]) free(emiss_curve->emissivity[i]);
  if (NULL != emiss_curve->emissivity) free(emiss_curve->emissivity);
  free(emiss_curve);

}


Emissivity_Curve_Type *emissivity_curve(Apec_Model *am,
					long element, 
					long ion, 
					long lowerlev, 
					long upperlev) {

  Emissivity_Curve_Type *tmp;
  long i, j, k;
  Line_Type *line;
  
  tmp = (Emissivity_Curve_Type *)malloc(sizeof(Emissivity_Curve_Type));

  tmp->nT = am->num_temp;
  tmp->ndensity = am->num_dens;
  
  tmp->kT = (float *)malloc(sizeof(float)*tmp->nT);  
  for (i=0;i<tmp->nT;i++) tmp->kT[i] = am->grid_kT[i];
  tmp->density = (float *)malloc(sizeof(float)*tmp->ndensity);
  for (i=0;i<tmp->ndensity;i++) tmp->density[i] = am->grid_density[i];

  tmp->emissivity = (float **)malloc(sizeof(float *)*tmp->nT);
  for (i=0;i<tmp->nT;i++) 
    tmp->emissivity[i] = (float *)malloc(sizeof(float)*tmp->ndensity);
  

  /* Now fill emissivity array */

  for (i=0;i<tmp->nT;i++) {
    for (j=0;j<tmp->ndensity;j++) {

      /* initialize */
      tmp->emissivity[i][j] = 0.0;



      /* find correct component */
      line = &(am->line[am->grid_kT_density[i][j]]);
      
      /* find the correct line */
      for (k=0;k<line->n;k++) {
	if (line->element[k] == element &&
	    line->ion[k] == ion &&
	    line->upperlev[k] == upperlev &&
	    line->lowerlev[k] == lowerlev) {
	  tmp->emissivity[i][j] = line->epsilon[k];
	}

      }
      
    }
  }


  return(tmp);    

}
