#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>
#include <fitsio.h>

#include "apec.h"
#include "_apec_slang.h"

int bisearch_lohi_f(int n, 
		    int inc,
		    float lambda, 
		    float *bin_lo, 
		    float *bin_hi) {

  int low, high, result=-1;

  low = 0;
  high = n-1;

  while ((high-low) > 1 ) {

    result = (low + high) / 2;
    if (inc) {
      if (lambda > bin_lo[result]) {
	low = result;
      } else {
	high = result;
      }
    } else {
      if (lambda < bin_lo[result]) {
	low = result;
      } else {
	high = result;
      }
    }
  }
  result--;

  if (result < 0) result = 0;
  /*
    while ((lambda - bin_lo[result])*(lambda - bin_hi[result]) > 0) {
    result++;
    if (result > Nbin-1) SLang_verror(SL_USER_ERROR, "Can't find lambda bin");
    }
  */
  return result;

}

int bisearch_f(int n, float value, float *array) {

  /* return index i of array element smaller or eqaul to value */

  int low, high, result=0;

  low = 0;
  high = n-1;

  while ((high-low) > 1 ) {

    result = (low + high) / 2;
    if (value > array[result]) {
      low = result;
    } else {
      high = result;
    }
  }

  result--;

  return(result);

}

int *find_Temperature_index(Apec_Model *am, float T, int *is) {

  /* 

     Use the specially constructed kt_density array to find the
     right index for the temperature.
  
  */

  if (T <= am->grid_kT[0]) {
    /* T is smaller that lowest temp in model; return lowest T */
    is[0] = 0;
    is[1] = 0;
  } else if (T >= am->grid_kT[am->num_temp-1]) {
    /* T is smaller that largest temp in model; return largest T */
    is[0] = am->num_temp-1;
    is[1] = am->num_temp-1;
  } else {
    /* Find bracketing temperatures. */
    is[0] = bisearch_f(am->num_temp,T,am->grid_kT);
    if (am->grid_kT[is[0]] == T) {
      is[1] = is[0];
    } else {
      is[1] = is[0]+1;
    }
  }

  return(is); 

}

int *find_Density_index(Apec_Model *am, float rho, int *is) {

  /* 

     Use the specially constructed kt_density array to find the
     right index for the temperature.
  
  */

  if (rho <= am->grid_density[0]) {
    /* rho is smaller that lowest density in model; return lowest density */
    is[0] = 0;
    is[1] = 0;
  } else if (rho >= am->grid_density[am->num_dens-1]) {
    /* rho is larger that largest density in model; return largest density */
    is[0] = am->num_dens-1;
    is[1] = am->num_dens-1;
  } else {
    /* Find bracketing densities. */
    is[0] = bisearch_f(am->num_dens,rho,am->grid_density);
    if (am->grid_density[is[0]] == rho) {
      is[1] = is[0];
    } else {
      is[1] = is[0]+1;
    }
  }

  return(is); 

}

int *find_grid_indices(Apec_Model *am, float T, float rho, int *is) {

  int i;

  for (i=0; i<4; i++) is[i] = -1;

  find_Density_index(am,rho,(is+2));
  find_Temperature_index(am,T,is);

  return(is);

}

int _extract_line_spectrum_(Line_Type *lines,
			    float *abundances,
			    float redshift,
			    int keV,
			    int inc,
			    float minLam,
			    float maxLam,
			    long nlinelist,
			    long *linelist,
			    int nbin, 
			    float *bin_lo,
			    float *bin_hi,
			    float *spec) {
  
  long i, pos;
  float lambda;
  long nline, *iline, *component_linelist;

  /* first make certain if we are dealing with a linelist ot not. */
  if (nlinelist != 0) {
    component_linelist = get_linelist_for_component(lines,nlinelist,linelist,&nline);
    iline = (long *)malloc(sizeof(long)*nline);
    for (i=0;i<nline;i++) iline[i] = component_linelist[i] - lines->linelist_start;
  } else {
    nline = lines->n;
    iline = (long *)malloc(sizeof(long)*nline);
    for (i=0;i<nline;i++) iline[i] = i;
  }

  /* loop over lines */
  for(i=0;i<nline;i++) {
    lambda = lines->lambda[iline[i]];
    /* Add redshift to wavelength */
    /* lambda *= (1.0+redshift); */
    if ((lambda >= minLam) && (lambda <= maxLam)) {
      /* Check is wavelength is in range of requested spectrum */
      if (keV) {
	/* Make sure we are dealing with the correct units */
	lambda = HC_KEV/lambda;  
      }
      /* Now find correct bin to include emissivity */
      pos = bisearch_lohi_f(nbin, inc, lambda, bin_lo, bin_hi);

      /*
	Should never happen!

	if ((bin_lo[pos] > lambda)||(bin_hi[pos] < lambda)) {
	SLang_verror (SL_USER_ERROR, "Bad position"); 
	}
      */
      
      spec[pos] += abundances[lines->element[iline[i]]]*
	           lines->epsilon[iline[i]];
      /*if (pos == 6) printf("%d %f %e %e\n", i, lambda, lines->epsilon[i], spec[pos]);*/
    }
    /*printf("%f %d %d\n",lambda, lines->element[iline[i]], lines->ion[iline[i]]); */
  }

  /*for(i=0;i<n;i++) if (spec[pos] > 1.0) printf("%d %f\n", i, spec[i]);*/

  free(iline);

  return(0);

}

int extract_line_spectrum(Apec_Model *am,
			  float kT, 
			  float density, 
			  float norm, 
			  float *abundances,
			  float redshift,
			  int keV, 
			  long nlinelist,
			  long *linelist,
			  int nbin, 
			  float *bin_lo,
			  float *bin_hi,
			  float *spectrum,
			  int *indx) {

  /* this routine always assumes both bin_lo and bin_hi and will return the integrated
     spectrum!
  */

  int inc, i, freeindx=0;
  float minLam, maxLam;
  float *spec1, *spec2, *spec3, *spec4;  /* dummy spectra */
  float T1, T2, R1, R2, w1, w2, w3, w4, diff;

  /* ascending/descending order ? */
  if (bin_lo[0] > bin_lo[1]) 
    inc = 0;
  else
    inc = 1;

  /* set spectrum to 0.0 */
  for (i=0;i<nbin;i++) { spectrum[i] = 0.0; }

  /* 
     first take care of the wavelength/energy units. 
  */

  if (inc) {
    if (keV) {
      minLam = HC_KEV/bin_hi[nbin-1];
      maxLam = HC_KEV/bin_lo[0];
    } else {
      minLam = bin_lo[0];
      maxLam = bin_hi[nbin-1];
    }
  } else {
    /* NOTE: Compared to Randall's version I switch lo and hi
       since for each lo/hi pair the value of lo should be 
       smaller that hi */
    if (keV) {
      minLam = HC_KEV/bin_hi[0];
      maxLam = HC_KEV/bin_lo[nbin-1];
    } else {
      minLam = bin_lo[nbin-1];
      maxLam = bin_hi[0];
    }
  }
  
  /* first find the correct point in the grid that corresponds
     to kT and density */
  
  if (indx == NULL) {
    freeindx = 1;
    indx = (int *)malloc(sizeof(int)*4);
    find_grid_indices(am,kT,density,indx);
  }

  /* now separate the different options */
  
  /* Consider the different possible interpolations */
  if (indx[0] == indx[1] && indx[2] == indx[3]) {
    if (-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spectrum)) {
	return(-1);
	}
  } else if (indx[2] == indx[3]) {
    /* No need to interpolate in density */
    T1 = am->grid_kT[indx[0]];
    T2 = am->grid_kT[indx[1]];
    diff = T2-T1;
    w1 = (T2 - kT)/diff;
    w2 = (kT - T1)/diff;
    spec1 = (float *)malloc(sizeof(float)*nbin);
    spec2 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec1[i] = 0.0; spec2[i] = 0.0; }
    if (-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[1]][indx[2]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec2)) {
      free(spec1); free(spec2);
      return(-1);
    }
    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w1*spec1[i] + 
	w2*spec2[i];
    }
    free(spec1);
    free(spec2);
  } else if (indx[0] == indx[1]) {
    /* No need to interpolate in temperature */
    R1 = am->grid_density[indx[2]];
    R2 = am->grid_density[indx[3]];
    diff = R2-R1;
    w3 = (R2 - density)/diff;
    w4 = (density - R1)/diff;
    spec3 = (float *)malloc(sizeof(float)*nbin);
    spec4 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec3[i] = 0.0; spec4[i] = 0.0; }
    if (-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[3]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec2)) {
      free(spec3); free(spec4);
      return(-1);
    }
    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w3*spec3[i] + 
	w4*spec4[i];
    }
    free(spec3);
    free(spec4);
  } else {
    /* bicubic interpolation in density and temperature */
    T1 = am->grid_kT[indx[0]];
    T2 = am->grid_kT[indx[1]];
    diff = T2-T1;
    w1 = (T2 - kT)/diff;
    w2 = (kT - T1)/diff;
    R1 = am->grid_density[indx[2]];
    R2 = am->grid_density[indx[3]];
    diff = R2-R1;
    w3 = (R2 - density)/diff;
    w4 = (density - R1)/diff;
    spec1 = (float *)malloc(sizeof(float)*nbin);
    spec2 = (float *)malloc(sizeof(float)*nbin);
    spec3 = (float *)malloc(sizeof(float)*nbin);
    spec4 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec1[i] = 0.0; spec2[i] = 0.0; 
                           spec3[i] = 0.0; spec4[i] = 0.0; }
    if (-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec1) ||
	-1 ==  _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[1]][indx[2]]]),
				       abundances,redshift,
				       keV,inc,minLam,maxLam,
				       nlinelist,linelist,
				       nbin,bin_lo,bin_hi,spec2) ||
	-1 ==  _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[0]][indx[3]]]),
				       abundances,redshift,
				       keV,inc,minLam,maxLam,
				       nlinelist,linelist,
				       nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_line_spectrum_(&(am->line[am->grid_kT_density[indx[1]][indx[3]]]),
				      abundances,redshift,
				      keV,inc,minLam,maxLam,
				      nlinelist,linelist,
				      nbin,bin_lo,bin_hi,spec2)) {
      free(spec1); free(spec2); free(spec3); free(spec4);
      return(-1);
    }
    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w1*w3*spec1[i] + 
	w2*w3*spec2[i] + 
	w1*w4*spec3[i] + 
	w2*w4*spec4[i];
    }

    free(spec1);
    free(spec2);
    free(spec3);
    free(spec4);
  }


  /*if (freeindx) {
    free(indx);
    indx = NULL;
    }*/

  /* Now that we have the spectrum, independent of normalization and 
     integration setting we can try to store it a for later reuse
     or not */
  

  /* apply normalization */
  for(i=0;i<nbin;i++) spectrum[i] *= 1.0e14*norm;

  return(0);

}


int _extract_coco_spectrum_(Coco_Type *coco,
			       float *abundances,
			       int keV,
			       int inc,
			       int nbin, 
			       float *bin_lo,
			       float *bin_hi,
			       float *spec) {
  
  /* The continuum is calculated in two steps: a) the real continuum
     and b) the Pseudo continuum
  */

  int i, j, k, l, pos, n;
  float emis;
  float *Energy_Array, *Emissivity_Array, *Slope_Array, *reverse;


  /* loop over elements */

  for (l=0;l<coco->n;l++) {

    /* loop for continuum types */

    for (j=0;j<2;j++) {
      if (j==0) {
	n = coco->n_cont[l];
	Energy_Array = (float *)malloc(sizeof(float)*n);
	Emissivity_Array = (float *)malloc(sizeof(float)*n);
	for (k=0;k<n;k++) {
	  Energy_Array[k] = coco->e_cont[l][k];
	  Emissivity_Array[k] = coco->continuum[l][k];
	}
      } else {
	n = coco->n_pseudo[l];
	Energy_Array = (float *)malloc(sizeof(float)*n);
	Emissivity_Array = (float *)malloc(sizeof(float)*n);
	for (k=0;k<n;k++) {
	  Energy_Array[k] = coco->e_pseudo[l][k];
	  Emissivity_Array[k] = coco->pseudo[l][k];
	}
      }
      Slope_Array = (float *)malloc(sizeof(float)*n);


      /* Make sure we are dealing with the correct units. If in energy
         then we are all set, but if in wavelength then we need to convert */
      /* Note:APEC data ALWAYS in increasing energy order */
      if (!keV) {  /* Units of ph cm^3 /s /keV */
	for (k=0;k<n;k++) {
	  Energy_Array[k] = HC_KEV/Energy_Array[k];
	  Emissivity_Array[k] *= HC_KEV/(Energy_Array[k]*Energy_Array[k]);
	}
	/* RH: CHECK WITH RANDALL IF THIS IS CORRECT */
	/* Now that we converted to wavelength the array is no longer
	   in increasing order. However, the algorithm below assumes 
	   a monotonically increasing array. So we need to convert this.
	*/
	reverse = (float *)malloc(sizeof(float)*n);
	for (k=0;k<n;k++) reverse[n-1-k] = Energy_Array[k];
	for (k=0;k<n;k++) Energy_Array[k] = reverse[k];
	for (k=0;k<n;k++) reverse[n-1-k] = Emissivity_Array[k];
	for (k=0;k<n;k++) Emissivity_Array[k] = reverse[k];
	free(reverse);

      }

      /* Do Slopes in one go */
      Slope_Array = (float *)malloc(sizeof(float)*n);
      for (k=0;k<n-1;k++) {
	Slope_Array[k] = (Emissivity_Array[k+1]-Emissivity_Array[k])/
	                 (Energy_Array[k+1]-Energy_Array[k]);
      }
      

      /* Emissivity is stored in fits file as a series of line slopes. Here
	 we reconstruct the spectrum by "integrating" over these sloped lines.
	 
	y|      2         \int\limits_{x_1}{x_2} y dx = 
	 |     /          y_1 * (x_2 - x_1) + 0.5 * (y_2 - y_1) * (x_2 - x_1)=
	 |    /           (2y_1 + y_2 - y_1)*(x_2-x_1)/2 = 
	 |   /            (y_1 + y_2) * (x_2 - x_1) / 2
	 |  1             
	 |                (same answer holds for declining line)
	 |___________x
	 
	 Of course we need to do some book keeping since the data x-axis bins
	 do not coincide with the bins if the stored spectra.

      */

      pos = 0;
      for (k=0;k<nbin;k++) {
	/* determine order of looping through bin_lo's */
	i = inc ? k : nbin-1-k;
	emis = 0.0;
	/* Damn, this assumes that Energy_Array is increasing order, but
	   (see above) if bin_lo/hi is in Angstrom this is no longer true. */
	while ((bin_lo[i] > Energy_Array[pos+1])&&(pos<n-1)) pos++;
	/* Now we have    |   |         |
                       E(pos) binlo     E(pos+1)
	*/
	if (pos < n-1) {
	  if (bin_hi[i] <= Energy_Array[pos+1]) {
	    /* bin_hi falls within this energy bin */
	    emis = 2*Emissivity_Array[pos] + Slope_Array[pos] *
	      ((bin_lo[i]-Energy_Array[pos]) + 
	       (bin_hi[i]-Energy_Array[pos]));
	    emis *= (bin_hi[i]-bin_lo[i])/2.; /* integrate over average */
	  } else {
	    emis = Emissivity_Array[pos+1] + Emissivity_Array[pos] + 
	           Slope_Array[pos]*(bin_lo[i]-Energy_Array[pos]);
	    emis *= (Energy_Array[pos+1]-bin_lo[i])/2;
	    pos++; /* Note: Slope value no longer accurate! */
	    while ((bin_hi[i] > Energy_Array[pos+1])&&(pos < n-1)) {
	      /* Find next energy bin bracketing binhi */
	      emis += (Emissivity_Array[pos+1] + Emissivity_Array[pos])*
		      (Energy_Array[pos+1]-Energy_Array[pos])/2.;
	      pos++;
	    }
	    if (pos < n-1) {
	      /* Add last part of bin */
	    emis += (2*Emissivity_Array[pos] + 
		     (bin_hi[i]-Energy_Array[pos])*Slope_Array[pos])*
	            (bin_hi[i]-Energy_Array[pos])/2.;
	    }
	  }
	}       
	spec[i] += abundances[coco->z[l]]*emis;
      }
      free(Energy_Array);
      free(Emissivity_Array);
      free(Slope_Array);
    }
    
  }

  /*for(i=0;i<n;i++) if (spec[pos] > 1.0) printf("%d %f\n", i, spec[i]);*/
  
  return(0);

}

int extract_coco_spectrum(Apec_Model *am,
			  float kT, 
			  float density, 
			  float norm, 
			  float *abundances,
			  int keV, 
			  int nbin, 
			  float *bin_lo,
			  float *bin_hi,
			  float *spectrum,
			  int *indx) {
  
  /* this routine always assumes both bin_lo and bin_hi and will 
     return the integrated spectrum! Note that the continuum is
     binned in energy and that the logical units are therefore the
     energy array and no the wavelength array.
  */

  int inc, i, freeindx = 0;
  float *spec1, *spec2, *spec3, *spec4;  /* dummy spectra */
  float T1, T2, R1, R2, w1, w2, w3, w4, diff;

  /* ascending/descending order ? */
  if (bin_lo[0] > bin_lo[1]) 
    inc = 0;
  else
    inc = 1;


  /* set spectrum to 0.0 */
  for (i=0;i<nbin;i++) { spectrum[i] = 0.0; }

  /* first find the correct point in the grid that corresponds
     to kT and density */
  
  if (indx == NULL) {
    freeindx=1;
    indx = (int *)malloc(sizeof(int)*4);
    find_grid_indices(am,kT,density,indx);
  }

  /* now separate the different options */

  /* Consider the different possible interpolations */
  if (indx[0] == indx[1] && indx[2] == indx[3]) {
    if (-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spectrum)) {
      return(-1);
    }
  } else if (indx[2] == indx[3]) {
    /* No need to interpolate in density */
    T1 = am->grid_kT[indx[0]];
    T2 = am->grid_kT[indx[1]];
    diff = T2-T1;
    w1 = (T2 - kT)/diff;
    w2 = (kT - T1)/diff;
    spec1 = (float *)malloc(sizeof(float)*nbin);
    spec2 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec1[i] = 0.0; spec2[i] = 0.0; }
    if (-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec1) ||
	-1 ==  _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[1]][indx[2]]]),
				       abundances,
				       keV,inc,nbin,bin_lo,bin_hi,spec2)) {
      free(spec1);
      free(spec2);
      return(-1);
    }
    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w1*spec1[i] + 
	w2*spec2[i];
    }
    free(spec1);
    free(spec2);
  } else if (indx[0] == indx[1]) {
    /* No need to interpolate in temperature */
    R1 = am->grid_density[indx[2]];
    R2 = am->grid_density[indx[3]];
    diff = R2-R1;
    w3 = (R2 - density)/diff;
    w4 = (density - R1)/diff;
    spec3 = (float *)malloc(sizeof(float)*nbin);
    spec4 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec3[i] = 0.0; spec4[i] = 0.0; }
    if (-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[3]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec2)) {
      free(spec3);
      free(spec4);
      return(-1);
    }
    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w3*spec3[i] + 
	w4*spec4[i];
    }
    free(spec3);
    free(spec4);
  } else {
    /* bicubic interpolation in density and temperature */
    T1 = am->grid_kT[indx[0]];
    T2 = am->grid_kT[indx[1]];
    diff = T2-T1;
    w1 = (T2 - kT)/diff;
    w2 = (kT - T1)/diff;
    R1 = am->grid_density[indx[2]];
    R2 = am->grid_density[indx[3]];
    diff = R2-R1;
    w3 = (R2 - density)/diff;
    w4 = (density - R1)/diff;
    spec1 = (float *)malloc(sizeof(float)*nbin);
    spec2 = (float *)malloc(sizeof(float)*nbin);
    spec3 = (float *)malloc(sizeof(float)*nbin);
    spec4 = (float *)malloc(sizeof(float)*nbin);
    for (i=0;i<nbin;i++) { spec1[i] = 0.0; spec2[i] = 0.0; 
    spec3[i] = 0.0; spec4[i] = 0.0; }
    if (-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[2]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[1]][indx[2]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec2) ||
	-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[0]][indx[3]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec1) ||
	-1 == _extract_coco_spectrum_(&(am->coco[am->grid_kT_density[indx[1]][indx[3]]]),
				      abundances,
				      keV,inc,nbin,bin_lo,bin_hi,spec2)) {
      free(spec1);
      free(spec2);
      free(spec3);
      free(spec4);
      return(-1);
    }

    for(i=0;i<nbin;i++) {
      spectrum[i] = 
	w1*w3*spec1[i] + 
	w2*w3*spec2[i] + 
	w1*w4*spec3[i] + 
	w2*w4*spec4[i];
    }

    free(spec1);
    free(spec2);
    free(spec3);
    free(spec4);
  }

  if (freeindx) {
    free(indx);
    indx = NULL;
  }

  /* Now that we have the spectrum, independent of normalization and 
     integration setting we can try to store it a for later reuse
     or not */

  /* apply normalization */
  for(i=0;i<nbin;i++) spectrum[i] *= 1.0e14*norm;

  return(0);

}


int extract_spectrum(Apec_Model *am,
		     float kT, 
		     float density, 
		     float norm, 
		     float *abundances,
		     float redshift,
		     int flag,
		     int keV, 
		     long nlinelist,
		     long *linelist,
		     int nbin, 
		     float *binlo,
		     float *binhi,
		     float *spectrum) {
  
  float *lines, *coco;
  float *rest_binlo, *rest_binhi, factor;
  float lorentz_gamma, x, y;
  int i, indx[4];

  if (am == NULL) {
    return(-1);
  }

  if (!((flag == LINES_ONLY) ||
	(flag == CONTINUUM_ONLY) ||
	(flag == LINES_AND_CONTINUUM))) {
    return(-1);
  }

  if (redshift < 0.0) {
    /* this is unphysical */
    return(-1);
  }

  if (redshift != 0.0) {
    rest_binlo = (float *)malloc(sizeof(float)*nbin);
    rest_binhi = (float *)malloc(sizeof(float)*nbin);
    if (keV == 0) {
      /* dealing with wavelengths */
      factor = 1.0 / (1.0 + redshift);
    } else {
      factor = 1.0 + redshift;
    }
    for (i=0;i<nbin;i++) {
      rest_binlo[i] = factor*binlo[i];
      rest_binhi[i] = factor*binhi[i];
    }
  } else {
    rest_binlo = binlo;
    rest_binhi = binhi;
  }


  if (flag == LINES_ONLY) {
    
    if (-1 == extract_line_spectrum(am,kT,density,norm,abundances,
				    redshift,keV,nlinelist,linelist,
				    nbin,rest_binlo,rest_binhi,
				    spectrum,NULL)) {
      free(rest_binlo);
      free(rest_binhi);
      return(-1);
    } 
  }

  if (flag == CONTINUUM_ONLY) {
    
    if (-1 == extract_coco_spectrum(am,kT,density,norm,abundances,
				    keV,nbin,rest_binlo,rest_binhi,
				    spectrum,NULL)) {
      free(rest_binlo);
      free(rest_binhi);
      return(-1);
    } 
  }

  
  if (flag == LINES_AND_CONTINUUM) {
    
    lines = (float *)malloc(sizeof(float)*nbin);
    coco = (float *)malloc(sizeof(float)*nbin);

    find_grid_indices(am,kT,density,indx);
    if (-1 == extract_line_spectrum(am,kT,density,norm,abundances,
				    redshift,keV,nlinelist,linelist,
				    nbin,rest_binlo,rest_binhi,
				    lines,indx) ||
	-1 == extract_coco_spectrum(am,kT,density,norm,abundances,
				    keV,nbin,rest_binlo,rest_binhi,
				    coco,indx)) {
      free(rest_binlo);
      free(rest_binhi);
      free(lines);
      free(coco);
      return(-1);
    }
    
    for (i=0;i<nbin;i++) spectrum[i] = lines[i]+coco[i];
    free(lines);
    free(coco);

  }

  if (redshift != 0.0) {
    free(rest_binlo);
    free(rest_binhi);

    /* We also need to take care of the time dilation effect since
       our units are in photon/sec/(A or keV)
       The effect of the per A or per keV units has been taken into
       account by changing the wavelenght/energy arrays 

       wvl_observed = wvl_rest * (1+z)
       
       and

       wvl_observed = wvl_rest * (1+y)/sqrt(1-y^2),  where y = v/c

       thus 

       x = (1+z) = (1+y)/sqrt(1-y^2) --->  y = (x^2-1)/(x^2+1)
       
       And the time dilation factor is

       t_observed = t_rest / gamma, with gamma = 1.0 / sqrt(1-y^2)

    */
    
    x = 1.0 + redshift;
    y = (x*x - 1.0)/(x*x + 1.0);     
    lorentz_gamma = 1.0/sqrt(1.0 - y*y);

    for (i=0;i<nbin;i++) spectrum[i] /= lorentz_gamma;

  }

  return(0);

}

int extract_ncomp_spectrum(Apec_Model *am,
			   int ncomp,
			   float *kT, 
			   float *density, 
			   float *norm, 
			   float *abundances,
			   float redshift,
			   int flag,
			   int keV, 
			   long nlinelist,
			   long *linelist,
			   int nbin, 
			   float *bin_lo,
			   float *bin_hi,
			   float *spectrum) {
  
  float *spec;
  int i;
  int icomp;

  spec = (float *)malloc(sizeof(float)*nbin);
  for(i=0;i<nbin;i++) {
    spectrum[i] = 0.0;
    spec[i] = 0.0;
  }


  for(icomp=0;icomp<ncomp;icomp++) {
    if (-1 == extract_spectrum(am,kT[icomp],density[icomp],norm[icomp],
			       abundances,redshift,flag,keV,
			       nlinelist,linelist,
			       nbin,bin_lo,bin_hi,spec)) {
      free(spec);
      return(-1);
    } else {
      for (i=0;i<nbin;i++) {
	spectrum[i] += spec[i];
	spec[i] = 0.0;
      }
    }
  }

  free(spec);

  return(0);

}

int extract_spectrum_for_modelname(char *name,
				   float kT, 
				   float density, 
				   float norm, 
				   float *abundances,
				   float redshift,
				   int flag,
				   int keV, 
				   long nlinelist,
				   long *linelist,
				   int nbin, 
				   float *bin_lo,
				   float *bin_hi,
				   float *spectrum) {
  
  Apec_Model *am;

  am = get_apec_model_with_name(name);

  if (-1 == extract_spectrum(am,kT,density,norm,abundances,
			     redshift,flag,keV,nlinelist,linelist,nbin, 
			     bin_lo,bin_hi,spectrum)) {
    return(-1);
  } else {
    return(0);
  }

}

int extract_ncomp_spectrum_for_modelname(char *name,
					 int ncomp,
					 float *kT, 
					 float *density, 
					 float *norm, 
					 float *abundances,
					 float redshift,
					 int flag,
					 int keV, 
					 long nlinelist,
					 long *linelist,
					 int nbin, 
					 float *bin_lo,
					 float *bin_hi,
					 float *spectrum) {

  Apec_Model *am;

  am = get_apec_model_with_name(name);
  
  if (-1 == extract_ncomp_spectrum(am,ncomp,kT,density,norm,abundances,
				   redshift,flag,keV,nlinelist,linelist,nbin, 
				   bin_lo,bin_hi,spectrum)) {
    return(-1);
  } else {
    return(0);
  }

}
