%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% model functions
%
define apec_powerlaw () {

   % input variables 
   variable gamma,ampl,density,nT,Tmin,Tmax,log_lin;               % 7
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 27
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable i, nargs = 27, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   nT = typecast(nT, Integer_Type);
   log_lin = typecast(log_lin, Integer_Type);

   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = em_power_law(modelname,spectrum_type,_apec_stuff_->linelist,
			  nT,Tmin,Tmax,log_lin,gamma,ampl,density,
			  abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}


%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define apec_broken_powerlaw () {

   % input variables 
   variable gamma1,gamma2,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin;% 9
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 29
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable i, nargs = 29, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma1,gamma2,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma1,gamma2,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   nT = typecast(nT, Integer_Type);
   log_lin = typecast(log_lin, Integer_Type);

   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = em_broken_power_law(modelname,spectrum_type,_apec_stuff_->linelist,
			  nT,Tmin,Tmax,log_lin,gamma1,gamma2,Tbreak,ampl,density,
			  abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}


%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define apec_double_powerlaw () {

   % input variables 
   variable gamma,beta,alpha,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin; % 10
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 30
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable i, nargs = 30, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma,beta,alpha,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (gamma,beta,alpha,Tbreak,ampl,density,nT,Tmin,Tmax,log_lin) = ();
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   nT = typecast(nT, Integer_Type);
   log_lin = typecast(log_lin, Integer_Type);

   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = em_double_power_law(modelname,spectrum_type,_apec_stuff_->linelist,
			  nT,Tmin,Tmax,log_lin,gamma,beta,alpha,Tbreak,ampl,density,
			  abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define apec_chebyshev () {

   % input variables 
   variable coeff,density,nT,Tmin,Tmax,log_lin;                    % 5+ncoeff
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 25+ncoeff
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable ncoeff = 10;
   variable i, nargs = 25+ncoeff, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (density,nT,Tmin,Tmax,log_lin) = ();
      coeff = array_from_args(__pop_args(ncoeff));
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (density,nT,Tmin,Tmax,log_lin) = ();
      coeff = array_from_args(__pop_args(ncoeff));
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   nT = typecast(nT, Integer_Type);
   log_lin = typecast(log_lin, Integer_Type);

   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = em_chebyshev(modelname,spectrum_type,_apec_stuff_->linelist,
			   nT,Tmin,Tmax,log_lin,coeff,density,
			   abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define apec_polynomial () {

   % input variables 
   variable coeff,density,nT,Tmin,Tmax,log_lin;                    % 5+ncoeff
   variable aHe,aC,aN,aO,aF,aNe,aMg,aAl,aSi,aS,aAr,aCa,aFe,aNi,    % 14
            abundances, nabundances = 14;
   variable redshift;                                              % 1
   variable modelname, str_spectrum_type, str_linelist;            % 3
   variable binlo, binhi;                                          % 2
                                                                   % --- +
                                                                   % 25+ncoeff
   
   % output variables
   variable spectrum;

   % auxilary parameters 
   variable ncoeff = 10;
   variable i, nargs = 25+ncoeff, inc, nbin, integrated;
   variable abundances_array, linelist, spectrum_type;
   variable savestack=NULL, dataset, curr_axis, keV=-1;

   %
   % First pop stuff from the stack
   %
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      usage("incorrect");
      _pop_n(_NARGS);
      return NULL;
   }

   %
   % Now distinguish between integrate ON and OFF
   %
   if (_NARGS == nargs-1) {
      % integrate off
      integrated = 0;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (density,nT,Tmin,Tmax,log_lin) = ();
      coeff = array_from_args(__pop_args(ncoeff));
      %
      % take care of inpbinhi
      %
      if (length(binlo) == 1) {
         return [0];
      } else if (binlo[1] < binlo[0]) {
         inc = 0;
      } else {
         inc = 1;
      }
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      nbin = length(binlo);
      if (inc) {
	 for(i=0;i<nbin-2;i++) binhi[i] = binlo[i+1];
	 binhi[nbin-1] = 2*binlo[nbin-1] - binlo[nbin-2];
      } else { 
	 for(i=1;i<nbin;i++) binhi[i] = binlo[i-1];
	 binhi[0] = 2*binlo[0] - binlo[1];
      }
   } else {
      % integrate on
      integrated = 1;
      %
      (redshift,modelname,str_spectrum_type,str_linelist,binlo,binhi) = ();
      abundances = array_from_args(__pop_args(nabundances));
      (density,nT,Tmin,Tmax,log_lin) = ();
      coeff = array_from_args(__pop_args(ncoeff));
      nbin = length(binlo);
   }

   %
   % Determine units of x-axis
   %
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   dataset = get_dataset_number();
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }

    if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strncmp(curr_axis.axisunits,"keV",3)==0) keV=1; % keV units
      if (strncmp(curr_axis.axisunits,"A",1)==0)   keV=0; % Angstrom units
      if (keV == -1) {
         message("Error: Input axis in neither keV nor Angstrom units.");
         return NULL;
      }
   }
 
   nT = typecast(nT, Integer_Type);
   log_lin = typecast(log_lin, Integer_Type);

   %vmessage("keV : %d",keV);

   %
   % Fix abundace array
   %
   abundances = make_abundance_array(abundances);

   %vmessage(" modelname : %s\n linelist : %s\n spectrum_type : %s",modelname,str_linelist, str_spectrum_type);

   %
   % get numeric value for spectrum_type
   %
   switch (str_spectrum_type)
   {(str_spectrum_type == "lines_and_continuum") or
    (str_spectrum_type == "LINES_AND_CONTINUUM") or
    (str_spectrum_type == "continuum_and_lines") or
    (str_spectrum_type == "CONTINUUM_AND_LINES") or
    (str_spectrum_type == "all") or
    (str_spectrum_type == "ALL") : spectrum_type = _apec_stuff_->_lines_and_continuum_; }
   {(str_spectrum_type == "lines_only") or
    (str_spectrum_type == "LINES_ONLY") or
    (str_spectrum_type == "lines") or
    (str_spectrum_type == "LINES") : spectrum_type = _apec_stuff_->_lines_; }
   {(str_spectrum_type == "continuum_only") or
    (str_spectrum_type == "CONTINUUM_ONLY") or
    (str_spectrum_type == "continuum") or
    (str_spectrum_type == "CONTINUUM") : spectrum_type = _apec_stuff_->_continuum_; }
   {vmessage("Did not recognize the value of spectrum_type (%s)\n --> will assume you meant lines_and_continuum...",str_spectrum_type);
    spectrum_type = _apec_stuff_->_lines_and_continuum_;}


   %
   % Get linelist. This is a bit of a dirty trick since arrays cannot
   % be passed as arguments to Sherpa functions. So we need to pass
   % the ciao variable, whose name is defined in the apec model to 
   % a global variable. We defined the namespace "_apec_stuff_" for this.
   %
   % IMPORTANT: TEMPORARY FIX UNTILL WE CAN CHANGE THE LINELIST PARAMETER
   %            IN SHERPA.
   if (str_linelist == "ball") {
      % no linelist defined and pass NULL to plasma call
      _apec_stuff_->linelist = NULL;
   } else {
      % make sure that we are referencing an existing variable
      if (-2 == is_defined("_ciao->"+str_linelist)) {
	 % link linelist to variable name.
	 eval("_apec_stuff_->linelist = _ciao->"+str_linelist);
      } else {
	 vmessage("Did not find variable named (%s)\n --> will continue using al lines to model the spectrum...",str_linelist);
	 _apec_stuff_->linelist = NULL;
      }
   }

   % IMPORTANT: TEMPORARY FIX TO MAKE SURE WE SEE ALL LINES.
   _apec_stuff_->linelist = NULL;
   %
   % OK now we actually calculate the spectrum.
   %
   spectrum = em_polynomial(modelname,spectrum_type,_apec_stuff_->linelist,
			    nT,Tmin,Tmax,log_lin,coeff,density,
			    abundances,redshift,keV,binlo,binhi);

   %spectrum = @binlo;
   %
   % OK, take care of integrate on/off
   %
   if (integrated == 0) {
      spectrum = spectrum / (binhi-binlo);   % to make it per Angstrom
   }

   %
   % We need to return the output in the correct type...
   % 
   if (_typeof(binlo) != _typeof(spectrum)) {
      spectrum = typecast(spectrum, _typeof(binlo));
   }

   return(spectrum);

}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
% Register the different models
%
%
define setup_em_models () {
    variable ncomp = 100;
    variable nfit_name = "nfit", 
             nfit_default = 1, 
             nfit_min = 1, 
             nfit_max=ncomp, 
             nfit_thaw = 0;
    variable kT_name = "kT", 
             kT_default = 1.0, 
             kT_min = 0.0001, 
             kT_max=80.0, 
             kT_thaw = 1;
    variable density_name = "density", 
             density_default = 1.0, 
             density_min = -4.0, 
             density_max= 20.0, 
             density_thaw = 0;
    variable norm_name = "norm", 
             norm_default = 1.0, 
             norm_min = 0.0, 
             norm_max= 100.0, 
             norm_thaw = 1;
    variable coeff0_name = "c0", 
             coeff0_default = 1.0, 
             coeff0_min =-100.0, 
             coeff0_max= 100.0, 
             coeff0_thaw = 1;
    variable coeff1_name = "c1", 
             coeff2_name = "c2", 
             coeff3_name = "c3", 
             coeff4_name = "c4", 
             coeff5_name = "c5", 
             coeff6_name = "c6", 
             coeff7_name = "c7", 
             coeff8_name = "c8", 
             coeff9_name = "c9", 
             coeffn_default = 0.0, 
             coeffn_min = coeff0_min, 
             coeffn_max = coeff0_max, 
             coeffn_thaw = 0;
    variable kT_default_nfit = kT_default, 
             kT_min_nfit = kT_min, 
             kT_max_nfit = kT_max, 
             kT_thaw_nfit = 0;
    variable density_default_nfit = density_default, 
             density_min_nfit = density_min, 
             density_max_nfit = density_max, 
             density_thaw_nfit = 0;
    variable norm_default_nfit = 0.0, 
             norm_min_nfit = norm_min, 
             norm_max_nfit = norm_max, 
             norm_thaw_nfit = 0;
    variable elements = ["He","C","N","0","F","Ne","Mg",
                         "Al","Si","S","Ar","Ca","Fe","Ni"];
    variable abundance_name = "abundance", 
             abundance_default = 1.0, 
             abundance_min = 0.0, 
             abundance_max=100.0, 
             abundance_thaw = 0;
    variable redshift_name = "redshift", 
             redshift_default = 0.0, 
             redshift_min = 0.0, 
             redshift_max=10.0, 
             redshift_thaw = 0;
    variable gamma_name = "gamma",
             gamma_default = 1.0, 
             gamma_min =-10.0, 
             gamma_max=10.0, 
             gamma_thaw = 1;
    variable beta_name = "beta",
             beta_default = gamma_default,
             beta_min = gamma_min,
             beta_max = gamma_max,
             beta_thaw = gamma_thaw;
    variable alpha_name = "alpha",
             alpha_default = gamma_default,
             alpha_min = gamma_min,
             alpha_max= gamma_max,
             alpha_thaw = gamma_thaw;
    variable gamma1_name = "gamma1",
             gamma1_default = gamma_default,
             gamma1_min = gamma_min,
             gamma1_max = gamma_max,
             gamma1_thaw = gamma_thaw;
    variable gamma2_name = "gamma2",
             gamma2_default = gamma_default,
             gamma2_min = gamma_min,
             gamma2_max= gamma_max,
             gamma2_thaw = gamma_thaw;
    variable Tbreak_name = "Tbreak",
             Tbreak_default = 1.0, 
             Tbreak_min =0.0001, 
             Tbreak_max=80.0, 
             Tbreak_thaw = 1;
    variable ampl_name = "ampl",
             ampl_default = 1.0, 
             ampl_min =0.0, 
             ampl_max=10.0, 
             ampl_thaw = 1;
    variable nT_name = "nT",
             nT_default = 10.0, 
             nT_min =1.0, 
             nT_max=100.0, 
             nT_thaw = 0;
    variable Tmin_name = "Tmin",
             Tmin_default = 0.1, 
             Tmin_min =0.0001, 
             Tmin_max=80.0, 
             Tmin_thaw = 0;
    variable Tmax_name = "Tmax",
             Tmax_default = 10.0, 
             Tmax_min =0.0001, 
             Tmax_max=80.0, 
             Tmax_thaw = 0;
    variable log_lin_name = "log_lin",
             log_lin_default = 0, 
             log_lin_min =0, 
             log_lin_max=1, 
             log_lin_thaw = 0;
    variable nstring = 3;
    variable modelname_name = "modelname", 
             modelname_default = "apec";
    variable spectrum_type_name = "spectrum_type", 
             spectrum_type_default = "lines_and_continuum";
    variable linelist_name = "linelist", 
             linelist_default = "all";
    variable par_name, par_default, par_min, par_max, par_thaw, par_string;
    variable i, j, narg;

    use_namespace("_ciao");

    % For the apec_simple model
    par_name    = [kT_name,density_name,norm_name,abundance_name,redshift_name,
                   modelname_name,spectrum_type_name,linelist_name];
    par_default = [kT_default,density_default,norm_default,abundance_default,redshift_default];
    par_min     = [kT_min,density_min,norm_min,abundance_min,redshift_min];
    par_max     = [kT_max,density_max,norm_max,abundance_max,redshift_max];
    par_thaw    = [kT_thaw,density_thaw,norm_thaw,abundance_thaw,redshift_thaw];
    par_string  = [modelname_default,spectrum_type_default,linelist_default];


    () = register_model("apec_powerlaw",
      [gamma_name,
       ampl_name,
       density_name,
       nT_name,
       Tmin_name,
       Tmax_name,
       log_lin_name,
       "abundance_He",
       "abundance_C",
       "abundance_N",
       "abundance_0",
       "abundance_F",
       "abundance_Ne",
       "abundance_Mg",
       "abundance_Al",
       "abundance_Si",
       "abundance_S",
       "abundance_Ar",
       "abundance_Ca",
       "abundance_Fe",
       "abundance_Ni",
       redshift_name,
       modelname_name,
       spectrum_type_name,
       linelist_name],
      1,
      [gamma_default,
       ampl_default,
       density_default,
       nT_default,
       Tmin_default,
       Tmax_default,
       log_lin_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       redshift_default],
      [gamma_min,
       ampl_min,
       density_min,
       nT_min,
       Tmin_min,
       Tmax_min,
       log_lin_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       redshift_min],
      [gamma_max,
       ampl_max,
       density_max,
       nT_max,
       Tmin_max,
       Tmax_max,
       log_lin_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       redshift_max],
      [gamma_thaw,
       ampl_thaw,
       density_thaw,
       nT_thaw,
       Tmin_thaw,
       Tmax_thaw,
       log_lin_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       redshift_thaw],
       par_string);	      

    () = register_model("apec_broken_powerlaw",
      [gamma1_name,
       gamma2_name,
       Tbreak_name,
       ampl_name,
       density_name,
       nT_name,
       Tmin_name,
       Tmax_name,
       log_lin_name,
       "abundance_He",
       "abundance_C",
       "abundance_N",
       "abundance_0",
       "abundance_F",
       "abundance_Ne",
       "abundance_Mg",
       "abundance_Al",
       "abundance_Si",
       "abundance_S",
       "abundance_Ar",
       "abundance_Ca",
       "abundance_Fe",
       "abundance_Ni",
       redshift_name,
       modelname_name,
       spectrum_type_name,
       linelist_name],
      1,
      [gamma1_default,
       gamma2_default,
       Tbreak_default,
       ampl_default,
       density_default,
       nT_default,
       Tmin_default,
       Tmax_default,
       log_lin_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       redshift_default],
      [gamma1_min,
       gamma2_min,
       Tbreak_min,
       ampl_min,
       density_min,
       nT_min,
       Tmin_min,
       Tmax_min,
       log_lin_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       redshift_min],
      [gamma1_max,
       gamma2_max,
       Tbreak_max,
       ampl_max,
       density_max,
       nT_max,
       Tmin_max,
       Tmax_max,
       log_lin_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       redshift_max],
      [gamma1_thaw,
       gamma2_thaw,
       Tbreak_thaw,
       ampl_thaw,
       density_thaw,
       nT_thaw,
       Tmin_thaw,
       Tmax_thaw,
       log_lin_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       redshift_thaw],
       par_string);	      

    () = register_model("apec_double_powerlaw",
      [gamma_name,
       beta_name,
       alpha_name,
       Tbreak_name,
       ampl_name,
       density_name,
       nT_name,
       Tmin_name,
       Tmax_name,
       log_lin_name,
       "abundance_He",
       "abundance_C",
       "abundance_N",
       "abundance_0",
       "abundance_F",
       "abundance_Ne",
       "abundance_Mg",
       "abundance_Al",
       "abundance_Si",
       "abundance_S",
       "abundance_Ar",
       "abundance_Ca",
       "abundance_Fe",
       "abundance_Ni",
       redshift_name,
       modelname_name,
       spectrum_type_name,
       linelist_name],
      1,
      [gamma_default,
       beta_default,
       alpha_default,
       Tbreak_default,
       ampl_default,
       density_default,
       nT_default,
       Tmin_default,
       Tmax_default,
       log_lin_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       redshift_default],
      [gamma_min,
       beta_min,
       alpha_min,
       Tbreak_min,
       ampl_min,
       density_min,
       nT_min,
       Tmin_min,
       Tmax_min,
       log_lin_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       redshift_min],
      [gamma_max,
       beta_max,
       alpha_max,
       Tbreak_max,
       ampl_max,
       density_max,
       nT_max,
       Tmin_max,
       Tmax_max,
       log_lin_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       redshift_max],
      [gamma_thaw,
       beta_thaw,
       alpha_thaw,
       Tbreak_thaw,
       ampl_thaw,
       density_thaw,
       nT_thaw,
       Tmin_thaw,
       Tmax_thaw,
       log_lin_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       redshift_thaw],
       par_string);	      

     () = register_model("apec_chebyshev",
      [coeff0_name,
       coeff1_name,
       coeff2_name,
       coeff3_name,
       coeff4_name,
       coeff5_name,
       coeff6_name,
       coeff7_name,
       coeff8_name,
       coeff9_name,
       density_name,
       nT_name,
       Tmin_name,
       Tmax_name,
       log_lin_name,
       "abundance_He",
       "abundance_C",
       "abundance_N",
       "abundance_0",
       "abundance_F",
       "abundance_Ne",
       "abundance_Mg",
       "abundance_Al",
       "abundance_Si",
       "abundance_S",
       "abundance_Ar",
       "abundance_Ca",
       "abundance_Fe",
       "abundance_Ni",
       redshift_name,
       modelname_name,
       spectrum_type_name,
       linelist_name],
      1,
      [coeff0_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       coeffn_default,
       density_default,
       nT_default,
       Tmin_default,
       Tmax_default,
       log_lin_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       abundance_default,
       redshift_default],
      [coeff0_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       coeffn_min,
       density_min,
       nT_min,
       Tmin_min,
       Tmax_min,
       log_lin_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       abundance_min,
       redshift_min],
      [coeff0_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       coeffn_max,
       density_max,
       nT_max,
       Tmin_max,
       Tmax_max,
       log_lin_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       abundance_max,
       redshift_max],
      [coeff0_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       coeffn_thaw,
       density_thaw,
       nT_thaw,
       Tmin_thaw,
       Tmax_thaw,
       log_lin_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       abundance_thaw,
       redshift_thaw],
       par_string);	      

      () = register_model("apec_polynomial",
       [coeff0_name,
        coeff1_name,
        coeff2_name,
        coeff3_name,
        coeff4_name,
        coeff5_name,
        coeff6_name,
        coeff7_name,
        coeff8_name,
        coeff9_name,
        density_name,
        nT_name,
        Tmin_name,
        Tmax_name,
        log_lin_name,
        "abundance_He",
        "abundance_C",
        "abundance_N",
        "abundance_0",
        "abundance_F",
        "abundance_Ne",
        "abundance_Mg",
        "abundance_Al",
        "abundance_Si",
        "abundance_S",
        "abundance_Ar",
        "abundance_Ca",
        "abundance_Fe",
        "abundance_Ni",
        redshift_name,
        modelname_name,
        spectrum_type_name,
        linelist_name],
       1,
       [coeff0_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        coeffn_default,
        density_default,
        nT_default,
        Tmin_default,
        Tmax_default,
        log_lin_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        abundance_default,
        redshift_default],
       [coeff0_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        coeffn_min,
        density_min,
        nT_min,
        Tmin_min,
        Tmax_min,
        log_lin_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        abundance_min,
        redshift_min],
       [coeff0_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        coeffn_max,
        density_max,
        nT_max,
        Tmin_max,
        Tmax_max,
        log_lin_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        abundance_max,
        redshift_max],
       [coeff0_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        coeffn_thaw,
        density_thaw,
        nT_thaw,
        Tmin_thaw,
        Tmax_thaw,
        log_lin_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        abundance_thaw,
        redshift_thaw],
        par_string);	      

 }
