define replace_envs();

define array_from_args() {

   if (_NARGS == 0) {
      error("array_from_args() called without arguments");
   }

   variable test = ();
   variable n = length(test), type;

   if (n > 0) {
      type = typeof(test[0].value);
   } else {
      return NULL;
   }
   
   variable array = @Array_Type(type,n);
   variable i;

   for (i=0;i<n;i++) array[i] = test[i].value;

   return array;

}

define position () {

   % 
   % Find the first occurence of 'elem' in the object 'str' with the
   % boundary conditions that we only look between 'start' and 'end'.
   % Mimics lisp function.
   %
   variable usage = "Usage: position(elem,string [,start [, end]])";
   variable i;
   variable elem, str, start = 0, end = NULL;

   if ((_NARGS != 2) and (_NARGS != 3) and (_NARGS != 4)) {
      message(usage);
   }

   switch (_NARGS) 
   { case 2: (elem, str) = (); }
   { case 3: (elem, str, start) = (); }
   { case 4: (elem, str, start, end) = (); }
   { message(usage); return; }

   if (NULL == end) {
      if (String_Type == typeof(str)) {
         end = strlen(str);
      } else if (Array_Type == typeof(str)) {
         end = length(str);
      }
   }

   for (i = start; i < end; i++) {
      if (elem == str[i]) return i;
   }
   
   return NULL;

}

define rposition () {

   % 
   % Find the first occurence of 'elem' in the object 'str' with the
   % boundary conditions that we only look between 'start' and 'end'.
   % search from the back to the front.
   % Mimics lisp function.
   %


   variable usage = "Usage: rposition(elem,string [,start [, end]])";
   variable i;
   variable elem, str, start = 0, end = NULL;

   if ((_NARGS != 2) and (_NARGS != 3) and (_NARGS != 4)) {
      message(usage);
   }


   switch (_NARGS) 
   { case 2: (elem, str) = (); }
   { case 3: (elem, str, start) = (); }
   { case 4: (elem, str, start, end) = (); }

   if (NULL == end) {
      if (String_Type == typeof(str)) {
         end = strlen(str);
      } else if (Array_Type == typeof(str)) {
         end = length(str);
      }
   }

   for (i = end-1; i >= start; i--) {
      if (elem == str[i]) return i;
   }
   
   return NULL;

}

define expand_env(FileName) {

   %
   %  Given a filename string which may (or may not) contain a 
   %  single environment variable, expand the environment variable
   %  and return the full filename.
   %

   variable expanded_FileName, envar, expanded_envar, envpos1, envpos2;
   
   if (FileName == NULL) {
      message("Call as: full_filename = expand_env(FileName)");
      return NULL;
   }
   
   envpos1 = string_match(FileName,"\\$",1);
   envpos2 = string_match(FileName,"/",envpos1);

   if (envpos1 == 0) {
      expanded_FileName = FileName;  % No environment variable to expand
   } else {
      if (envpos2 == 0) {
	 envar = FileName[[envpos1-1:]];
      } else {
	 envar = FileName[[envpos1-1:envpos1-1+envpos2-2]];
      }
      expanded_envar = getenv(envar[[1:]]);
      if (expanded_envar == NULL) {
	 vmessage("expand_env: Environment variable %s in string %s undefined",
		  envar, FileName);
	 return NULL;
      }
      %	 vmessage("envar=%s",envar);
      
      (expanded_FileName,) = str_replace(FileName, envar, expanded_envar);
   }
   return expanded_FileName;
}


define replace_envs() {

   variable str, penv, env, slash;

   switch(_NARGS)
   { case 1 : str = (); penv = 0;}
   { error("incorrect number of arguments to replace_envs"); }

   if (is_defined("expand_env") == 0) () = evalfile("expand_env.sl");

   if (penv == 0) {
      penv = position('\$',str);
   } else {
      penv = position('\$',str,penv+1);
   }

   if (penv != NULL) {

      slash = position('/',str,penv);
      if (NULL == slash) slash = position('\$',str,penv+1);
      if (NULL == slash) {
         env = str[[penv:]];
         if (penv == 0) {
            str = expand_env(env);
         } else {
            str = str[[0:penv-1]] + expand_env(env);
         }
      } else {
         env = str[[penv:slash-1]];
         if (penv == 0) {
            str = expand_env(env) + str[[slash:]];
         } else {
            str = str[[0:penv-1]] + expand_env(env) + str[[slash:]];
         }
      }
   }

   if (position('\$',str)) return replace_envs(str);

   return str;
   
}


define member (obj, array) {

   if (_typeof(obj) != _typeof(array)) {
      message("In routine \"member(obj,array)\":"); 
      vmessage("  Data types of object and array do not match ");
      return NULL;
   }


   variable x = where(array == obj);

   if (length(x) == 0) {
      return NULL;
   } else {
      return x[0];
   }

}

define probe_file() {

   variable usage = "Usage: probe_file(filename [,dirname])";
   variable filename, dirname = NULL;
   variable slash, ls;

   switch(_NARGS)
   { case 1 : filename = (); dirname = "."; }
   { case 2 : (filename, dirname) = (); }
   { message(usage); return NULL; }
    
   %
   % Get rid of any environmental variables
   %
   filename = replace_envs(filename);
   dirname = replace_envs(dirname);

   slash = rposition('/',filename);

   if (slash != NULL) {
      if (dirname == ".") {
         % extract dirname
         dirname = filename[[0:slash-1]];
      } else {
         dirname = dirname + filename[[0:slash-1]];
      }
   } else {
      slash = -1;
   }
   filename = filename[[slash+1:]];
   
   ls = listdir(dirname);
   if (member(filename,ls) != NULL) {
      return 1;
   } else {
      return 0;
   }

}

