/** \file DataImport.h
    \brief Declaration of DataImport and related classes.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_DataImport_h
#define profit_DataImport_h

#include "RootFrame.h"

class TGTransientFrame;

#include "RQ_OBJECT.h"
#include "Rtypes.h"

namespace profit {

  class Gui;

  /** \class DataImport
      \brief Class which creates/managers data import modal dialog box, for interpreting
      data fields after they have been selected from a file using FileImport.
  */
  class DataImport {
    public:
      RQ_OBJECT("profit::DataImport")
      ClassDef(DataImport, 0)

      /** \brief Create a DataImport dialog box, associated with the given Gui object.
          \param gui The top-level gui.
      */
      DataImport(Gui * gui);

      virtual ~DataImport();

      /** \brief For testing purposes only, run in test mode, which causes the dialog box
          to be displayed, then various methods exercised, rather than just displaying the dialog box.
          TODO: Note this doesn't really work correctly.
      */
      virtual void runInTestMode();

      /// \brief Respond to close window event by safely dismantling the dialog box.
      virtual void handleCloseWindow();

    private:
      /// \brief Construct Root's primitive Gui frame for this dialog box.
      TGTransientFrame * buildGui();

      RootFrameTracker m_tracker;
      Gui * m_gui;
  };

}

#endif
