/** \file AtomDbImp.h
    \brief Declaration of AtomDbImp class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_AtomDbImp_h
#define profit_AtomDbImp_h

#include "profit/AtomDb.h"
#include "profit/AtomicLine.h"

namespace profit {

  class IEnergy;

  /** \class AtomDbImp
      \brief Class encapsulating access to the atomic database.
  */
  class AtomDbImp {
    public:
      typedef AtomDb::AtomicLineCont AtomicLineCont;

      virtual ~AtomDbImp() {}

      /** \brief Discard database entries (in memory) whose energy falls outside an energy range.
          \param min_energy Energy object with minimum value of range.
          \param max_energy Energy object with maximum value of range.
      */
      virtual void filter(const IEnergy & min_energy, const IEnergy & max_energy) = 0;

      /** \brief Discard database entries (in memory) whose emissivity is less than the given threshold.
          \param min_emissivity Minimum emissivity accepted.
      */
      virtual void filter(double min_emissivity) = 0;

      /// \brief Get container of atomic lines which match the current criteria for line selection.
      virtual const AtomicLineCont & getMatches() const = 0;
  };

}

#endif
