/** \file DataDataHandler.cxx
    \brief Implementation of DataHandler class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Datum.h"
#include "profit/DataHandler.h"
#include "profit/DataView.h"

namespace profit {

  DataHandler::~DataHandler() {}

  void DataHandler::select(Datum & datum, data_cont_type & deselect) {
  }

  void DataHandler::handleMessage(const Message & message, const Datum & datum) {
    // Create container to store pointers to data affected by this message.
    const_data_cont_type affected_data;

    // Perform specific handling of this message, tracking all data which is modified in the affected_data container.
    handle(message, datum, affected_data);

    // Build up a container of all views affected by data affected by the handle method.
    Datum::view_cont_type affected_view;
    for (const_data_cont_type::iterator itor = affected_data.begin(); itor != affected_data.end(); ++itor) {
      (*itor)->getAffectedViews(affected_view);
    }

    // Update all views which were indirectly affected by this action, passing as a hint the
    // container of affected data, in case this helps the views update their states more efficiently.
    for (view_cont_type::iterator itor = affected_view.begin(); itor != affected_view.end(); ++itor) {
      (*itor)->update(affected_data);
    }
  }

}
