/** \file Energy.cxx
    \brief Implementation of energy-related abstractions.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Energy.h"

namespace profit {

  //  Product of Planck's constant and speed of light, in keV-Angstroms, used to convert energy to wavelength and vice versa.
  const double IEnergy::s_h_c = 12.3984;

  // Small but non-0 value to protect from division by 0.
  const double IEnergy::s_epsilon = 1.e-15;

  // Limits derived from the values above.
  const double IEnergy::s_min_energy = IEnergy::s_epsilon;
  const double IEnergy::s_max_energy = 1. / IEnergy::s_epsilon;
  const double IEnergy::s_min_wavelength = IEnergy::s_h_c / IEnergy::s_max_energy;
  const double IEnergy::s_max_wavelength = IEnergy::s_h_c / IEnergy::s_min_energy;
}
