/** \file ProfitApplication.cxx
    \brief Implementation of ProfitApplication class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Gui.h"
#include "profit/ProfitApplication.h"

#include <sstream>

namespace {

  std::string s_cvs_id("$Name: 2.0 $");

}

namespace profit {

  ProfitApplication::ProfitApplication(): m_gui(Gui::instance()) {
    if (!s_arg.empty()) m_gui->openSpecFile(s_arg.front());

    std::string cvs_blank_id("$Name: ");
    if (s_cvs_id == cvs_blank_id + " $") {
      m_version = "develop";
    } else {
      // String to hold the version, with cvs $Name stuff stripped out.
      m_version = s_cvs_id.substr(cvs_blank_id.size(), s_cvs_id.size() - cvs_blank_id.size() - 2);

      // Convert tag from v1r2p3 format into 1.2.3 format.
      char vers_char;
      char rev_char;
      char patch_char;
      int vers_number;
      int rev_number;
      int patch_number;
      std::string remainder;

      // Use stream to parse version string into version, revision, patch and any remainder after the patch.
      std::stringstream ss;
      ss << m_version;
      ss >> vers_char >> vers_number >> rev_char >> rev_number >> patch_char >> patch_number >> remainder;

      if ('v' == vers_char && 'r' == rev_char && 'p' == patch_char) {
        std::stringstream ss2;
        ss2 << vers_number << "." << rev_number << "." << patch_number << remainder;
        ss2 >> m_version;
      }
    }
  }

  ProfitApplication::~ProfitApplication() {}

  void ProfitApplication::runApp() { m_gui->setVersion(m_version); m_gui->run(); }

}
