/** \file RootFrame.h
    \brief Declaration of RootFrame class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_RootFrame_h
#define profit_RootFrame_h

#include "TGLayout.h"

#include <list>
#include <string>

class TGCompositeFrame;
class TGFrame;
class TGLayoutHints;

namespace profit {

  /** \class RootFrame
      \brief Class wrapping a Root primitive frame, providing some convenient memory management.
  */
  class RootFrame {
    public:
      RootFrame(TGFrame * frame, TGCompositeFrame * parent = 0, ULong_t hints = kLHintsNormal, Int_t pad_left = 0,
        Int_t pad_right = 0, Int_t pad_top = 0, Int_t pad_bottom = 0);

      RootFrame(const RootFrame & frame);

      virtual ~RootFrame();

      TGLayoutHints * getHints();

      bool operator ==(const TGFrame * frame) const;

    private:
      RootFrame & operator =(const RootFrame & frame);
      void assign(RootFrame & frame);

      TGFrame * m_frame;
      TGCompositeFrame * m_parent;
      TGLayoutHints * m_layout_hints;
  };

  class RootFrameTracker {
    public:
      typedef std::list<RootFrame> frame_cont_type;

      ~RootFrameTracker();

      void trackFrame(const std::string & context, TGFrame * frame, TGCompositeFrame * parent, ULong_t hints = kLHintsNormal,
        Int_t pad_left = 0, Int_t pad_right = 0, Int_t pad_top = 0, Int_t pad_bottom = 0);

      void clear();

      RootFrame & front();

      RootFrame & back();

    private:
      frame_cont_type m_frame_cont;
  };

}

#endif
