/** \file Stream.cxx
    \brief Implementation of stream-related abstractions.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Stream.h"

#include <limits>

namespace profit {

  // Global debug mode flag.
  static bool s_debug_mode = false;

  // Name of the current executable.
  static std::string s_exec_name;

  // Global chatter maximum.
  static unsigned int s_global_max_chat = std::numeric_limits<unsigned int>::max();

  void InitStdStreams(const std::string & exec_name, unsigned int max_chat, bool debug_mode) {
    // Perform initialization only once.
    static bool s_init_done = false;

    if (!s_init_done) {
      // Initialize perr, plog and pout.
      OStream::initStdStreams();

      // Set global parameters affecting stream output.
      s_debug_mode = debug_mode;
      s_exec_name = exec_name;
      s_global_max_chat = max_chat;
      s_init_done = true;
    }
  }

  bool GetDebugMode() {
    return s_debug_mode;
  }

  const std::string & GetExecName() {
    return s_exec_name;
  }

  unsigned int GetMaximumChatter() {
    return s_global_max_chat;
  }

  void SetDebugMode(bool debug_mode) {
    s_debug_mode = debug_mode;
  }

  void SetExecName(const std::string & exec_name) {
    s_exec_name = exec_name;
  }

  void SetMaximumChatter(unsigned int max_chat) {
    s_global_max_chat = max_chat;
  }

}
