/** \file profit_main.cxx
    \brief Main profit application.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Application.h"

int main(int argc, char ** argv) {
  using namespace profit;

  // Handle command line to set up application properly.
  Application::parseCommandLine(argc, argv);

  // Create the main profit application object.
  Application * app = Application::create();

  // Assume failure (status != 0) until success is achieved.
  int status = 1;

  if (0 != app) {
    // Run the application.
    app->run();

    // If no failure indicated, change exit status to 0 (success).
    if (!app->failed()) status = 0;

    // Clean up.
    delete app;
  }

  // Report final status.
  return status;
}
